#!/usr/bin/python

#
# gpsdll2mh.py
#
##########################################################################
# G1OGY 20081102  Initial release
#
# G1OGY 20090419: Updated to cope with increased precision of gpsd v2.39
#
##########################################################################
#
# `gpsd' [http://gpsd.berlios.de] console-client which prints Maidenhead 
# LOCATOR at user-determined, arbitary, even-number precision.
#
# (LOCATOR precision max=52 - it gets quite silly long before that).
#
# This program is intended for 'Rover-style' operation (or WAB, perhaps)
# so that one's 'new' locator can be sent as soon as neccessary.
# It will also assist in calculating a (static) locator to a greater
# precision for microwave dish aiming purposes.
#
##########################################################################
#
# Command line args are supported to select 
# Locator precision (default=6)
# repeat frequency 
# server:port 
# Requires: access to a `gpsd' daemon
#
# Command line (console) usage:
#
#       python gpsdll2mh.py [[[precision] [repeat]] [server:port]]] 
#
##########################################################################
#
# Credits:
# The Lat/Long to Maidenhead part of this program is a rework of an 
# original script by Dan Jacobson -- http://jidanni.org/geo/maidenhead/ ;
# latterly updated by Rene Kanter.
#
## lonlat2maidenhead -- long/lat to Maidenhead grid calculator not limited
## to 6 characters.
## Copyright       : http://www.fsf.org/copyleft/gpl.html
## Author          : Dan Jacobson -- http://jidanni.org/geo/maidenhead/
## Created On      : Sat Mar 15 03:54:08 2003
## rkanters 2004.2.20 version ll2mh
#
# Thank you, gentlemen.
#
##########################################################################
#

# get modules
import re, sys, string
from time import sleep
from socket import *

# Set variables (defaults)
gpsdHost = 'localhost'
gpsdPort = 2947                         # gpsd default (IANA)
gpsdCall = ['p']                        # fixed gpsd command for 'position'

# print usage at default start ('cos of my cheap way to do cmd-line args parse)
print '\n***********************************************************************'
print '\n',sys.argv[0],
if len(sys.argv)<2:
    print "Running : 6-char LOC, 5 sec rptr, `gpsd' local, default \n"
    print 'Usage:'
    print '      ',sys.argv[0],'[[[LOC Precision] [Repeat delay]] [Server:port]]]\n'
# Abort!
print '\nINFO :: ** Ctrl-C to stop **\n\n'

# validate the input precision value: even number
if len(sys.argv)>1:
    loclen=int(sys.argv[1])
    if loclen<2 or loclen%2!=0:
        sys.stderr.write('ERROR :: Locator precision requested must be an even number \n\n')
        sys.exit(87)
else:
    loclen=6                            # why report only Square? (it won't often change unless you're airborne!)
                                        # if only Square is needed: then '4'
maxn=loclen/2

# grab or set repeat value
if len(sys.argv)>2:
    snooze=int(sys.argv[2])
else:
    snooze = 5                          # seconds


# gpsd network server
if len(sys.argv)>3:
     sp = sys.argv[3].split(":")
#    print sp				# deebug
     gpsdHost=sp[0]
     gpsdPort=int(sp[1])
print 'INFO :: Connecting to :  ', gpsdHost,':',gpsdPort,'\n'

# Networking
# create local socket
gpsdSock = socket(AF_INET, SOCK_STREAM)

# connect to gpsd daemon
try:
    gpsdSock.connect((gpsdHost, gpsdPort))
except Exception, e:
    print " FATAL ERROR :: Cannot connect to `gpsd'!\n"
    print "Is `gpsd' running? Is your server:port address correct?\n"
    print "System message ::" , e ,'\n'
    
#sleep(15)


# send position query, receive response and calculate LOCATOR
# ~persevere~ if daemon is recalcitrant
try:
    while 1:
        for Query in gpsdCall:
            gpsdSock.send(Query)
            data = gpsdSock.recv(33)	# increased to cope with 9-figure mantissa from gpsd v2.39
            if len(data)<12:
                break
            latlong = data[7:]

            A=ord('A')                  # set a base value for the calcs.
            
# Following commented code allowed for manual entry of the lat/long values - 
# maybe I'll revisit this sometime.
#            
## while 1:
##    line=sys.stdin.re adline()
##    if not line: break
##    ll=re.findall(r'([-0-9.]+)\s+([-0-9.]+)',line)
#

            ll=re.findall(r'([-0-9.]+)\s+([-0-9.]+)',latlong)
            if ll:
                for x,y in ll:
                    lat=float(x)
                    lon=float(y)
            else:
                sys.stderr.write(sys.argv[0]+': ERROR :: Cannot determine LAT / LONG. Is ''`gpsd'' running?\n\n')
                sys.exit(44)
                if -180<=lon<180: pass
                else:
                    sys.stderr.write('ERROR :: longitude must be -180<=lon<180\n\n')
                    sys.exit(32)
                
            if -90<=lat<90: pass
            else:
                sys.stderr.write('ERROR :: latitude must be -90<=lat<90\n\n')
                sys.exit(33)                    # can't handle north pole, sorry, [A-R]
                
            lon=(lon+180.0)/20                  # scale down and set up for first digit
            lat=(lat+90.0)/10
            mhloc=""
            i=0
            while i<maxn:
                i+=1
                loni=int(lon)
                lati=int(lat)
                if i%2:
                    mhloc+=chr(A+loni)+chr(A+lati)
                    lon=(lon-loni)*10
                    lat=(lat-lati)*10
                else:
                    mhloc+=str(loni)+str(lati)
                    lon=(lon-loni)*24
                    lat=(lat-lati)*24
            
# print lat/long 'cos a short (<12) locator won't update much 
# (unless you've got the hammer down)                 

            print 'Lat / Long: ', latlong, 'Locator:    ', mhloc, '\n'
            sleep(snooze)

# Close down            

except KeyboardInterrupt:                       # inhibits the traceback on exit
    gpsdSock.shutdown(1)
    gpsdSock.close()
            
# returns the grid square, to the precision given, that contains the given point.
#


# TO DO
# DONE! Trap no valid GPS Data - if GPS device is unplugged gpsd continues but provides `?'
# also exposed a strange sensation where telnet rtned data but this prog (without the `break')
# ONLY received `?'
# DONE! Trap no gpsd daemon - can't connect
# Esoteric...
    # keep count (and display on close) of all Fields [IO, JO, JN], Squares [JO01, IO91, JP63] 
    # and Sub-Squares [JO01GR, JO01HR] traversed on the journey.
    # 3 counters inside. Storage??  mySQL-Lite??
    # print 'em ???
